/**************************************************************

	Kontrola pravopisu 

**************************************************************/
#include <windows.h>
#include <stdio.h>
#pragma hdrstop
#include "m32rc.h"


/******************************************

	MemFile 
	- soubor mapovany do virtualni pameti... 

******************************************/
class CMemFile
{
protected:
	HANDLE		hFile;
	HANDLE		hFileMapping;

public:
	CMemFile(LPSTR name);
	virtual ~CMemFile();

	int			size;	
	char		*ptr;	// ukazatel na obsah souboru
};

CMemFile::CMemFile(LPSTR name)
{
	hFile			= INVALID_HANDLE_VALUE;
	hFileMapping	= NULL;
	ptr				= NULL;

	hFile = CreateFile( name, GENERIC_READ, FILE_SHARE_READ, 
		NULL, OPEN_EXISTING, NULL, NULL);	// otevru jen pro cteni
	if(hFile == INVALID_HANDLE_VALUE) return;
		
	size = GetFileSize(hFile, NULL);

	hFileMapping = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, size, NULL);
	if(!hFileMapping) return;

	ptr = (char *)MapViewOfFile(hFileMapping, FILE_MAP_READ, 0, 0, size);
	if(!ptr) return;
}; 

CMemFile::~CMemFile() 
{
	if(ptr)	
		UnmapViewOfFile(ptr);
	if(hFileMapping)
		CloseHandle(hFileMapping);
	if(hFile != INVALID_HANDLE_VALUE)
		CloseHandle(hFile);
}


/******************************************

	Slovnik

******************************************/
class CSlovnik : public CMemFile
{
public:
	bool isOK() { return( ptr ? TRUE : FALSE );	}
	bool checkWord(char *word);
	CSlovnik() : CMemFile("dict.txt") { }
};

/*
 *	checkWord -> je slovo ve slovniku ?
 */
bool CSlovnik::checkWord(char *word)
{
	char	*end;
	int		i;
	UINT	len;

	i = 0;
	while(i < size) {
		/*	najdu znak \ nebo konec radky */

		end = strchr(ptr + i, '//');
		if(!end) {
			end = strchr(ptr + i, '\n');
			if(!end) end = ptr;
		}
		len = end - ptr - i;	// spoctu delku slova na radku

		/* porovnam slova */

		if( ( strlen(word) == len ) && 
			( memcmp(ptr + i, word, len) == 0 ) ) return(TRUE);

		/* skocim da dalsi radek */
		end = strchr(ptr + i, '\n');
		i = end - ptr + 1;
	}

	return(FALSE);
}

/******************************************

	Word - trida pro prochazeni textem

******************************************/
class CWord : public CMemFile
{	
	static char *space;

	bool	isSpace(char chr);
public:
	int		pos;
	int		len;
	char	*getWord(char *buf);

	CWord(char *file) : CMemFile(file) { pos = len = 0; }
	void reset() { pos = 0; }
};

/* oddelovaci znaky */
char *CWord::space = " \t\n\r,./;'[]~!@#$%^&*()_+";

/*
 *	je znak mezera nebo pismeno ? 
 */
bool CWord::isSpace(char chr) {
	for(UINT i = 0;i < strlen(space);i++) 
		if(chr == space[i]) return(TRUE);

	return(FALSE);
}		

/*
 *	getWord - najde slovo v textu
 */
char *CWord::getWord(char *buf) {
	int		end;

	if( pos == size ) {
		buf[0] = 0;
		return(NULL);
	}
		
	pos += len;

	/* presunu se na prvni pismeno */
	while( ( isSpace(ptr[pos]) ) && ( pos < size ) ) pos++;
	if( pos == size ) {
		buf[0] = 0;
		return(NULL);
	}

	/* najdu konec */
	end = pos;
	while( ( !isSpace(ptr[end]) ) && ( end < size ) ) end++; 
	
	len = end - pos;

	memcpy(buf, ptr+pos, len);
	buf[len] = 0;

	return(ptr + pos);
}

void process(char *in, char *out, bool interactive) 
{
	CSlovnik	dict;
	CWord		word(in);
	FILE		*f = fopen(out, "w+");

	char		buf[100];
	int			start;
	bool		stop = FALSE;

	start = 0;
	while(word.getWord(buf)) {

		/* preskocim spravna slova	*/
		while( dict.checkWord(buf) ) if(!word.getWord(buf)) break;

		fwrite(word.ptr + start, 1, word.pos - start, f);

		/* zapisu to spatne			*/
		fwrite("#", 1, 1, f);
		fwrite(word.ptr + word.pos, 1, word.len, f);

		start = word.pos + word.len;
	}
}

/***********************************************************************************

		Funkce grafickeho rozhrani

***********************************************************************************/
char outName[MAX_PATH] = "";
char inName[MAX_PATH] = "";

/*
 *	procedura hlavniho dialogu 
 */
BOOL CALLBACK DlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg) {
	case WM_INITDIALOG: return(TRUE);
	case WM_COMMAND:
		switch(wParam) {
		case IDC_INP_BROWSE:
			{
				OPENFILENAME ofn;
				memset(&ofn, 0, sizeof(ofn));
				ofn.lStructSize			= sizeof(ofn); 

				ofn.lpstrFilter 		= "*.TXT\0";
			    ofn.hwndOwner			= hWnd; 
			    ofn.lpstrFile			= inName; 
				ofn.nMaxFile			= MAX_PATH; 
				ofn.Flags				= OFN_FILEMUSTEXIST;
				ofn.lpstrDefExt			= "txt";
				GetOpenFileName(&ofn);

				SetDlgItemText(hWnd, IDC_INPUT, inName);
			} break;
		case IDC_OUT_BROWSE: 
			{
				OPENFILENAME ofn;
				memset(&ofn, 0, sizeof(ofn));
				ofn.lStructSize			= sizeof(ofn); 

			    ofn.hwndOwner			= hWnd; 
			    ofn.lpstrFile			= outName; 
				ofn.lpstrFilter 		= "*.TXT\0";
				ofn.nMaxFile			= MAX_PATH; 

				GetSaveFileName(&ofn);
				SetDlgItemText(hWnd, IDC_OUTPUT, outName);
			} break;
		case IDC_PROCESS:
			process(inName, outName, TRUE);
			MessageBox(hWnd, "Kontrola probhla...", "Kontrola pravopisu", MB_OK);
			break;
		case IDC_QUIT: EndDialog(hWnd, TRUE);break;
		} break;
	case WM_CLOSE: EndDialog(hWnd, TRUE);break;
	}
	return(FALSE);
}


/*
 *	procedura dialogu nahrazeni
 */
BOOL CALLBACK ReplaceProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg) {
	case WM_INITDIALOG: return(TRUE);
	case WM_CLOSE:	EndDialog(hWnd, TRUE);break;

	}
	return(FALSE);
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR parm1, int) 
{
	char *parm2;

	/* najdu druhy parametr */
	parm2 = parm1;
	while( (*parm2 != 0) && (*parm2 != ' ') ) parm2++;

	if(parm2 != parm1) {
		*parm2 = 0;
		parm2++;

		process(parm1, parm2, FALSE);
	}
	else {
		DialogBox(hInstance, "MAINDLG", NULL, (DLGPROC)DlgProc);
	}

	return(0);
}